# -*- coding: utf-8 -*-
"""
/***************************************************************************
 ArrayPlus
                                 A QGIS plugin
 Functions to manipulate arrays in field calculator
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2018-06-06
        git sha              : $Format:%H$
        copyright            : (C) 2018 by JB Desbas
        email                : jb.desbas@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt5.QtCore import QSettings, QTranslator, qVersion, QCoreApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QAction

from qgis.utils import iface,qgsfunction
from qgis.core import QgsExpression

# Initialize Qt resources from file resources.py
#from .resources import *
# Import the code for the dialog
from .array_plus_dialog import ArrayPlusDialog
import os.path

@qgsfunction(args='auto', group='Arrays')
def array_min(arr, feature, parent):
    """
    Return the min value in an array
    <h4>Syntax</h4>
    <strong>array_min</strong>(<i>array</i>)
    """
    return min(arr)

@qgsfunction(args='auto', group='Arrays')
def array_max(arr, feature, parent):
    """
    Return the max value in an array
    <h4>Syntax</h4>
    <strong>array_max</strong>(<i>array</i>)
    """
    return max(arr)

@qgsfunction(args='auto', group='Arrays')
def array_sort(arr, feature, parent):
    """
    Sort the values by ascending order (use array_reverse for desc)
    <h4>Syntax</h4>
    <strong>array_sort</strong>(<i>array</i>)
    """
    arr.sort()
    return arr

@qgsfunction(args='auto', group='Arrays')
def array_avg(arr, feature, parent):
    """
    Calculate the avg of the values
    <h4>Syntax</h4>
    array_avg(<i>array</i>)
    """
    arr=list(map(float,arr))
    return sum(arr)/len(arr)

@qgsfunction(args='auto', group='Arrays')
def array_sum(arr, feature, parent):
    """
    Calculate the sum of the values
    <h4>Syntax</h4>
    <strong>array_sum</strong>(<i>array</i>)
    """
    arr=list(map(float,arr))
    return sum(arr)

@qgsfunction(args='auto', group='Arrays')
def array_count(arr, value, feature, parent):
	"""
	Count a given value in an array
    <h4>Syntax</h4>
    <strong>array_count</strong>(<i>array</i>,<i>value</i>)
	<h4>Example usage</h4>
	<ul>
	 <li>array_count(string_to_array( 'a,b,c,a,a'),'a')->3</li>
	 </ul>
	"""
	from collections import Counter
	cnt=Counter(arr)
	return cnt[value]
	
@qgsfunction(args='auto', group='Arrays')
def array_majority(arr, feature, parent):
    """
    Return the most common value. Return arbitrary value if ex-aequo
    <h4>Syntax</h4>
    <strong>array_majority</strong>(<i>array</i>)
    <h4>Example usage</h4>
	<ul>
    <li>array_majority(string_to_array( 'a,b,c,a,a'))->'a'</li>
    </ul>
    """
    from collections import Counter
    cnt=Counter(arr)
    return cnt.most_common()[0][0]

@qgsfunction(args='auto', group='Arrays')
def array_minority(arr, feature, parent):
    """
    Return the most common value. Return arbitrary value if ex-aequo
    <h4>Syntax</h4>
    <strong>array_minority</strong>(<i>array</i>)
    <h4>Example usage</h4>
    <ul>
    <li>array_minority(string_to_array( 'a,b,c,a,a'))->'c'</li>
    </ul>
    """
    from collections import Counter
    cnt=Counter(arr)
    return cnt.most_common()[-1][0]

@qgsfunction(args='auto', group='Arrays')
def array_replace(arr, old, new, feature, parent):
    """
    Replace old value by new value
    <h4>Syntax</h4>
    <strong>array_minority</strong>(<i>array</i>,<i>before</i>,<i>after</i>)
    """
    for k,e in enumerate(arr):
        if e==old:
            arr[k]=new
    return arr
    
@qgsfunction(args='auto', group='Arrays')
def array_lambda(arr, lmbd, feature, parent):
    """
    Apply a custom fonction to each element (x)
    <h4>Syntax</h4>
    <p><strong>array_lambda</strong>(<i>array</i>, <i>lambda function</i>)
    <h4>Example usage</h4>
    <p><strong>array_lambda</strong>(array(1,2,3),'x+10') -> <i>&lt;array: 11,12,13&gt;</i>
    """
    out=list(map(lambda x: eval(lmbd),arr))
    return out

@qgsfunction(args='auto', group='Maps')
def hstore_to_map(string, feature, parent):
    ##TODO : regex
    """
    Convert hstore string to map
    <h4>Syntax</h4>
    <strong>hstore_to_map</strong>(<i>string</i>)
    """
    out={}
    rows=string.replace('"',"").split(',')
    for e in rows:
        e=e.split('=>')
        out.update({e[0]:e[1]})
    return out
    
@qgsfunction(args='auto', group='Maps')    
def json_to_map(string, feature, parent):
    """
    Convert json string to map 
    <h4>Syntax</h4>
    <p><strong>json_to_map</strong>(<i>json string</i>)
    <h4>Example usage</h4>
    <p>json_to_map('{"a":12,"b":["rre","rra"]}') -> <i>&lt;map: a:12,&lt;array: 'rre','rra'&gt;&gt;</i>
    """
    import json
    return json.loads(string)
    
class ArrayPlus:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'ArrayPlus_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('ArrayPlus', message)

    def initGui(self):
        QgsExpression.registerFunction(array_min)
        QgsExpression.registerFunction(array_max)
        QgsExpression.registerFunction(array_minority)
        QgsExpression.registerFunction(array_majority)
        QgsExpression.registerFunction(array_count)
        QgsExpression.registerFunction(array_avg)
        QgsExpression.registerFunction(array_sum)
        QgsExpression.registerFunction(array_sort)
        QgsExpression.registerFunction(array_replace)
        QgsExpression.registerFunction(array_lambda)
        QgsExpression.registerFunction(hstore_to_map)
        QgsExpression.registerFunction(json_to_map)


    def unload(self):
        QgsExpression.unregisterFunction('array_min')
        QgsExpression.unregisterFunction('array_max')
        QgsExpression.unregisterFunction('array_minority')
        QgsExpression.unregisterFunction('array_majority')
        QgsExpression.unregisterFunction('array_count')
        QgsExpression.unregisterFunction('array_avg')
        QgsExpression.unregisterFunction('array_sum')
        QgsExpression.unregisterFunction('array_sort')
        QgsExpression.unregisterFunction('array_replace')
        QgsExpression.unregisterFunction('array_lambda')
        QgsExpression.unregisterFunction('hstore_to_map')
        QgsExpression.unregisterFunction('json_to_map')
